@@ -9,22 +9,19 @@ fi
 case $BUILDSYSTEM in
 
     *static*)
-        if [[ $OSTYPE == darwin* ]]; then
-            install -m 0644 $(brew --prefix)/opt/libomp/lib/libomp.dylib ${PREFIX}/bin
-            otool -L ${PREFIX}/bin/normaliz
-            install_name_tool -id "@loader_path/./libomp.dylib" ${PREFIX}/bin/libomp.dylib
-            install_name_tool -change "$(brew --prefix)/opt/libomp/lib/libomp.dylib" "@loader_path/./libomp.dylib" ${PREFIX}/bin/normaliz
-           otool -L ${PREFIX}/bin/normaliz
-         fi
+        ## export NORMPARA=-x=1 ## if paralleization should not work
 
+        make install
         if [[ $OSTYPE == darwin* ]]; then
+            echo "WWWWWWWWWWWWWWWWWWWWWW"
+            diff --version
+            echo "VVVVVVVVVVVVVVVVVVVVVV"
             otool -L ${PREFIX}/bin/*
+            echo "UUUUUUUUUUUUUUUUUUUUUU"
         else
             ldd ${PREFIX}/bin/*
         fi
 
-        ## export NORMPARA=-x=1 ## paralleization does not work at present
-
         make check
 
         cd local/bin
