@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -39,6 +39,7 @@
 #include "libnormaliz/vector_operations.h"
 #include "libnormaliz/list_and_map_operations.h"
 #include "libnormaliz/automorph.h"
+#include "libnormaliz/fusion.h"
 
 namespace libnormaliz {
 using namespace std;
@@ -150,7 +151,8 @@ void Output<Integer>::setCone(Cone<Integer>& C) {
         else
             monoid_or_cone = "cone";
         of_polyhedron = " of polyhedron (homogenized)";
-        if ((Result->isComputed(ConeProperty::ModuleGenerators) || Result->isComputed(ConeProperty::NumberLatticePoints)) &&
+
+        if ((Result->isComputed(ConeProperty::ModuleGenerators) || Result->isComputed(ConeProperty::NumberLatticePoints) || Result->isComputed(ConeProperty::SingleLatticePoint)) &&
             Result->getRecessionRank() == 0) {
             if (!using_renf<Integer>())
                 module_generators_name = " lattice points in polytope (module generators)" + polynomial_constraints;
@@ -164,6 +166,8 @@ void Output<Integer>::setCone(Cone<Integer>& C) {
                 module_generators_name = " module generators";
         }
     }
+    if(Result->isComputed(ConeProperty::SingleLatticePoint) && !Result->isComputed(ConeProperty::NumberLatticePoints))
+        module_generators_name += " (only single lattice point asked for)";
 }
 
 template <typename Number>
@@ -333,7 +337,7 @@ void Output<Integer>::write_matrix_mod(const Matrix<Integer>& M) const {
 
 template <typename Integer>
 void Output<Integer>::write_matrix_lat(const Matrix<Integer>& M) const {
-    if (ext == true) {
+    if (lat == true) {
         M.print(name, "lat");
     }
 }
@@ -807,17 +811,17 @@ void Output<Integer>::write_inv_file() const {
             inv << "integer number_vertices_polyhedron = " << Result->getNrVerticesOfPolyhedron() << endl;
         }
         if (Result->isComputed(ConeProperty::MarkovBasis)) {
-            inv << "integer number_markov_basis_elemrnts = " << Result->getNrMarkovBasis() << endl;
+            inv << "integer number_markov_basis_elements = " << Result->getNrMarkovBasis() << endl;
         }
         if (Result->isComputed(ConeProperty::GroebnerBasis)) {
-            inv << "integer number_groebner_basis_elemrnts = " << Result->getNrGroebnerBasis() << endl;
+            inv << "integer number_groebner_basis_elements = " << Result->getNrGroebnerBasis() << endl;
         }
         if (Result->isComputed(ConeProperty::Representations)) {
             inv << "integer number_representations = " << Result->getNrRepresentations() << endl;
             vector<key_t> KeyForOutput = Result-> getHilbertBasisKey();
             for(size_t i = 0; i < KeyForOutput.size(); ++i)
                 KeyForOutput[i]++;
-            inv << "vector " << KeyForOutput.size() << "hilbert_basis_key = "
+            inv << "vector " << KeyForOutput.size() << " hilbert_basis_key = "
             << KeyForOutput << endl;
         }
         if (Result->isComputed(ConeProperty::ExtremeRays)) {
@@ -834,6 +838,13 @@ void Output<Integer>::write_inv_file() const {
         if (Result->isComputed(ConeProperty::DualFVector)) {
             inv << "vector " << Result->getDualFVector().size() << " dual_f_vector = " << Result->getDualFVector();
         }
+        if (Result->isComputed(ConeProperty::FVectorOrbits)) {
+            inv << "vector " << Result->getFVectorOrbits().size() << " f_vector_orbits = " << Result->getFVectorOrbits();
+        }
+        if (Result->isComputed(ConeProperty::DualFVectorOrbits)) {
+            inv << "vector " << Result->getDualFVectorOrbits().size() << " dual_f_vector_orbits = "
+                     << Result->getDualFVectorOrbits();
+        }
         if (Result->isComputed(ConeProperty::MaximalSubspace)) {
             size_t dim_max_subspace = Result->getDimMaximalSubspace();
             inv << "integer dim_max_subspace = " << dim_max_subspace << endl;
@@ -897,13 +908,11 @@ void Output<Integer>::write_inv_file() const {
             inv << "vector " << Result->getAxesScaling().size() << " axes_scaling " << Result->getAxesScaling();
         if (Result->isComputed(ConeProperty::Grading) == false) {
             if (Result->isComputed(ConeProperty::ExtremeRays)) {
-                inv << "boolean graded = "
-                    << "false" << endl;
+                inv << "boolean graded = " << "false" << endl;
             }
         }
         else {
-            inv << "boolean graded = "
-                << "true" << endl;
+            inv << "boolean graded = " << "true" << endl;
             if (!Result->isComputed(ConeProperty::Dehomogenization) && Result->isComputed(ConeProperty::NumberLatticePoints)) {
                 inv << "integer degree_1_elements = " << Result->getNumberLatticePoints() << endl;
             }
@@ -914,19 +923,29 @@ void Output<Integer>::write_inv_file() const {
             }
             inv << endl;
             inv << "integer grading_denom = " << Result->getGradingDenom() << endl;
-            if (Result->isComputed(ConeProperty::Multiplicity)) {
-                mpq_class mult = Result->getMultiplicity();
-                inv << "integer multiplicity = " << mult.get_num() << endl;
-                inv << "integer multiplicity_denom = " << mult.get_den() << endl;
-            }
-            if (Result->isComputed(ConeProperty::Volume) && !using_renf<Integer>()) {
-                mpq_class vol = Result->getVolume();
-                inv << "integer volume = " << vol.get_num() << endl;
-                inv << "integer volume_denom = " << vol.get_den() << endl;
-            }
-            if (Result->isComputed(ConeProperty::WeightedEhrhartSeries)) {
-                const HilbertSeries& HS = Result->getIntData().getWeightedEhrhartSeries().first;
+        }
+        if (Result->isComputed(ConeProperty::Multiplicity)) {
+            mpq_class mult = Result->getMultiplicity();
+            inv << "integer multiplicity_num = " << mult.get_num() << endl;
+            inv << "integer multiplicity_denom = " << mult.get_den() << endl;
+        }
+        if (Result->isComputed(ConeProperty::Volume) && !using_renf<Integer>()) {
+            mpq_class vol = Result->getVolume();
+            inv << "integer volume_num = " << vol.get_num() << endl;
+            inv << "integer volume_denom = " << vol.get_den() << endl;
+        }
+        if (Result->isComputed(ConeProperty::WeightedEhrhartSeries)) {
+            const HilbertSeries& HS = Result->getIntData().getWeightedEhrhartSeries().first;
 
+            vector<mpz_class> HSCyclonum = HS.getCyclotomicNum();
+            vector<denom_t> HSCyclodenom = to_vector(HS.getCyclotomicDenom());
+
+            inv << "vector " << HSCyclonum.size() << " weighted_ehrhart_series_cyclo_num = ";
+            inv << HSCyclonum;
+            inv << "vector " << HSCyclodenom.size() << " weighted_ehrhart_series_cyclo_denom = ";
+            inv << HSCyclodenom;
+
+                if(!HS.get_only_cyclotomic()){
                 inv << "vector " << HS.getNum().size() << " weighted_ehrhart_series_num = " << HS.getNum();
                 inv << "integer num_common_denom = " << Result->getIntData().getWeightedEhrhartSeries().second << endl;
                 inv << "vector " << to_vector(HS.getDenom()).size()
@@ -948,30 +967,38 @@ void Output<Integer>::write_inv_file() const {
                     inv << "integer expansion_coeff_common_denom = " << Result->getIntData().getWeightedEhrhartSeries().second
                         << endl;
                 }
-            }
+            } // cyclonomic
+        }
 
-            if (Result->isComputed(ConeProperty::VirtualMultiplicity)) {
-                mpq_class mult = Result->getVirtualMultiplicity();
-                inv << "integer virtual_multiplicity = " << mult.get_num() << endl;
-                inv << "integer virtual_multiplicity_denom = " << mult.get_den() << endl;
+        if (Result->isComputed(ConeProperty::VirtualMultiplicity)) {
+            mpq_class mult = Result->getVirtualMultiplicity();
+            inv << "integer virtual_multiplicity_num = " << mult.get_num() << endl;
+            inv << "integer virtual_multiplicity_denom = " << mult.get_den() << endl;
+        }
+        if (Result->isComputed(ConeProperty::Integral)) {
+            mpq_class mult = Result->getIntegral();
+            inv << "integer integral_num = " << mult.get_num() << endl;
+            inv << "integer integral_denom = " << mult.get_den() << endl;
+        }
+        if (Result->isComputed(ConeProperty::HilbertSeries)) {
+            const HilbertSeries& HS = Result->getHilbertSeries();
+            vector<mpz_class> HSnum;
+            vector<denom_t> HSdenom;
+            vector<mpz_class> HSCyclonum = HS.getCyclotomicNum();
+            vector<denom_t> HSCyclodenom = to_vector(HS.getCyclotomicDenom());
+            if (Result->isComputed(ConeProperty::HSOP)) {
+                HSnum = HS.getHSOPNum();
+                HSdenom = to_vector(HS.getHSOPDenom());
             }
-            if (Result->isComputed(ConeProperty::Integral)) {
-                mpq_class mult = Result->getIntegral();
-                inv << "integer integral = " << mult.get_num() << endl;
-                inv << "integer integral_denom = " << mult.get_den() << endl;
+            else {
+                HSnum = HS.getNum();
+                HSdenom = to_vector(HS.getDenom());
             }
-            if (Result->isComputed(ConeProperty::HilbertSeries)) {
-                const HilbertSeries& HS = Result->getHilbertSeries();
-                vector<mpz_class> HSnum;
-                vector<denom_t> HSdenom;
-                if (Result->isComputed(ConeProperty::HSOP)) {
-                    HSnum = HS.getHSOPNum();
-                    HSdenom = to_vector(HS.getHSOPDenom());
-                }
-                else {
-                    HSnum = HS.getNum();
-                    HSdenom = to_vector(HS.getDenom());
-                }
+            inv << "vector " << HSCyclonum.size() << " hilbert_series_cyclo_num = ";
+            inv << HSCyclonum;
+            inv << "vector " << HSCyclodenom.size() << " hilbert_series_cyclo_denom = ";
+            inv << HSCyclodenom;
+            if(!HS.get_only_cyclotomic()){
                 inv << "vector " << HSnum.size() << " hilbert_series_num = ";
                 inv << HSnum;
                 inv << "vector " << HSdenom.size() << " hilbert_series_denom = ";
@@ -991,7 +1018,8 @@ void Output<Integer>::write_inv_file() const {
                     inv << "vector Hilbert_series_expansion " << expansion.size() << " = " << expansion;
                 }
             }
-        }
+        } // cyclotomic
+
         if (Result->isComputed(ConeProperty::IsReesPrimary)) {
             if (Result->isReesPrimary()) {
                 inv << "boolean primary = true" << endl;
@@ -1037,19 +1065,37 @@ void Output<Integer>::write_inv_file() const {
 template <typename Integer>
 void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const {
     HilbertSeries HS = Result->getIntData().getWeightedEhrhartSeries().first;
-    out << "Weighted Ehrhart series:" << endl;
-    vector<mpz_class> num(HS.getNum());
-    for (const auto& i : num)
-        out << i << " ";
-    out << endl << "Common denominator of coefficients: ";
-    out << Result->getIntData().getWeightedEhrhartSeries().second << endl;
-    map<long, long> HS_Denom = HS.getDenom();
-    long nr_factors = 0;
-    for (const auto& it : HS_Denom) {
-        nr_factors += it.second;
+
+    // vector<mpz_class> HS_Num;
+    map<long, long> HS_Denom;
+    bool only_cyclotomic = HS.get_only_cyclotomic();
+    bool no_quasipol = !HS.get_quasipol_allowed();
+    long period = HS.getPeriod();
+
+    bool write_series = !only_cyclotomic;
+    bool write_cyclo = only_cyclotomic;
+    bool write_polynomial = !only_cyclotomic && period == 1 && (HS_Denom.size() == 0 || HS_Denom.begin()->first == (long)HS_Denom.size());
+    bool write_quasi_pol = !write_polynomial;
+    write_polynomial &= !no_quasipol;
+    write_cyclo = write_cyclo || !write_polynomial;
+    write_polynomial &= !no_quasipol;
+    write_quasi_pol &= !no_quasipol;
+
+    if(write_series){
+        out << "Weighted Ehrhart series:" << endl;
+        vector<mpz_class> num(HS.getNum());
+        for (const auto& i : num)
+            out << i << " ";
+        out << endl << "Common denominator of coefficients: ";
+        out << Result->getIntData().getWeightedEhrhartSeries().second << endl;
+        map<long, long> HS_Denom = HS.getDenom();
+        long nr_factors = 0;
+        for (const auto& it : HS_Denom) {
+            nr_factors += it.second;
+        }
+        out << "Series denominator with " << nr_factors << " factors:" << endl;
+        out << HS.getDenom();
     }
-    out << "Series denominator with " << nr_factors << " factors:" << endl;
-    out << HS.getDenom();
     if (HS.getShift() != 0) {
         out << "shift = " << HS.getShift() << endl << endl;
     }
@@ -1065,19 +1111,26 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const {
         out << endl;
     }
 
-    long period = HS.getPeriod();
-    if (period == 1) {
-        out << "Weighted Ehrhart polynomial:" << endl;
-        for (const auto& i : HS.getHilbertQuasiPolynomial()[0])
-            out << i << " ";
-        out << endl;
-        out << "with common denominator = ";
-        out << HS.getHilbertQuasiPolynomialDenom() * Result->getIntData().getNumeratorCommonDenom();
+    if(write_polynomial){
+        if (period == 1) {
+            out << "Weighted Ehrhart polynomial:" << endl;
+            for (const auto& i : HS.getHilbertQuasiPolynomial()[0])
+                out << i << " ";
+            out << endl;
+            out << "with common denominator = ";
+            out << HS.getHilbertQuasiPolynomialDenom() * Result->getIntData().getNumeratorCommonDenom() << endl << endl;
+
+            long deg = HS.getHilbertQuasiPolynomial()[0].size() - 1;
+            out << "Degree of (quasi)polynomial: " << deg << endl;
+
+            long virtDeg = Result->getRank() + Result->getIntData().getDegreeOfPolynomial() - 1;
+            out << endl << "Expected degree = " << virtDeg << endl;
+        }
     }
-    else {
+    if(write_cyclo) {
         // output cyclonomic representation
         out << "Weighted Ehrhart series with cyclotomic denominator:" << endl;
-        num = HS.getCyclotomicNum();
+        vector<mpz_class> num = HS.getCyclotomicNum();
         for (const auto& i : num)
             out << i << " ";
         out << endl << "Common denominator of coefficients = ";
@@ -1085,6 +1138,8 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const {
         out << "Series cyclotomic denominator:" << endl;
         out << HS.getCyclotomicDenom();
         out << endl;
+    }
+    if(write_quasi_pol){
         // Weighted Ehrhart quasi-polynomial
         // vector< vector<mpz_class> > hilbert_quasi_poly = HS.getHilbertQuasiPolynomial();
         if (HS.isHilbertQuasiPolynomialComputed()) {
@@ -1095,21 +1150,19 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const {
             }
             Matrix<mpz_class> HQP(HS.getHilbertQuasiPolynomial());
             HQP.pretty_print(out, true);
-            out << "with common denominator: " << Result->getIntData().getWeightedEhrhartQuasiPolynomialDenom() << endl;
+            out << "with common denominator: " << Result->getIntData().getWeightedEhrhartQuasiPolynomialDenom() << endl << endl;
+
+            long deg = HS.getHilbertQuasiPolynomial()[0].size() - 1;
+            out << "Degree of (quasi)polynomial: " << deg << endl;
+
+            long virtDeg = Result->getRank() + Result->getIntData().getDegreeOfPolynomial() - 1;
+            out << endl << "Expected degree = " << virtDeg << endl;
         }
-        else {
-            out << "Weighted Ehrhart quasi-polynomial has period " << period << endl;
+        else{
+                out << "Weighted Ehrhart quasi-polynomial has period " << period << endl;
         }
-    }
 
-    out << endl << endl;
-
-    if (HS.isHilbertQuasiPolynomialComputed()) {
-        long deg = HS.getHilbertQuasiPolynomial()[0].size() - 1;
-        out << "Degree of (quasi)polynomial: " << deg << endl;
-
-        long virtDeg = Result->getRank() + Result->getIntData().getDegreeOfPolynomial() - 1;
-        out << endl << "Expected degree = " << virtDeg << endl;
+        out << endl;
     }
 
     if (Result->isComputed(ConeProperty::VirtualMultiplicity)) {
@@ -1117,7 +1170,7 @@ void Output<Integer>::writeWeightedEhrhartSeries(ofstream& out) const {
         if (Result->isComputed(ConeProperty::FixedPrecision))
             virtual_mult_string += " (fixed precision)";
         virtual_mult_string += " = ";
-        out << endl << virtual_mult_string;
+        out <<endl << virtual_mult_string;
         out << Result->getIntData().getVirtualMultiplicity() << endl;
         if (Result->getIntData().getVirtualMultiplicity().get_den() != 1)
             out << "Virtual multiplicity (float) = " << std::setprecision(12)
@@ -1132,57 +1185,76 @@ template <typename Integer>
 void Output<Integer>::writeSeries(ofstream& out, const HilbertSeries& HS, string HilbertOrEhrhart) const {
     vector<mpz_class> HS_Num;
     map<long, long> HS_Denom;
-    if (Result->isComputed(ConeProperty::HSOP)) {
-        HS_Denom = HS.getHSOPDenom();
-        HS_Num = HS.getHSOPNum();
-        string HSOP;
-        if (!HS_Denom.empty())  // we disable the HSOP attribute if the series is a polynomial
-            HSOP = " (HSOP)";
-        out << HilbertOrEhrhart << "series" << HSOP << ":" << endl << HS_Num;
-    }
-    else {
-        HS_Denom = HS.getDenom();
-        HS_Num = HS.getNum();
-        out << HilbertOrEhrhart + "series:" << endl << HS_Num;
-    }
-    long nr_factors = 0;
-    for (auto& it : HS_Denom) {
-        nr_factors += it.second;
-    }
-    out << "denominator with " << nr_factors << " factors:" << endl;
-    out << HS_Denom;
-    out << endl;
-    if (HS.getShift() != 0) {
-        out << "shift = " << HS.getShift() << endl << endl;
+    bool only_cyclotomic = HS.get_only_cyclotomic();
+    bool no_quasipol = !HS.get_quasipol_allowed();
+    long period = HS.getPeriod();
+
+    bool write_series = !only_cyclotomic;
+    bool write_cyclo = only_cyclotomic;
+    bool write_polynomial = !only_cyclotomic && period == 1 && (HS_Denom.size() == 0 || HS_Denom.begin()->first == (long)HS_Denom.size());
+    bool write_quasi_pol = !write_polynomial;
+    write_cyclo = write_cyclo || write_quasi_pol;
+    write_polynomial &= !no_quasipol;
+    write_quasi_pol &= !no_quasipol;
+
+    if(write_series){
+        if (Result->isComputed(ConeProperty::HSOP)) {
+            HS_Denom = HS.getHSOPDenom();
+            HS_Num = HS.getHSOPNum();
+            string HSOP;
+            if (!HS_Denom.empty())  // we disable the HSOP attribute if the series is a polynomial
+                HSOP = " (HSOP)";
+            out << HilbertOrEhrhart << "series" << HSOP << ":" << endl << HS_Num;
+        }
+        else {
+            HS_Denom = HS.getDenom();
+            HS_Num = HS.getNum();
+            out << HilbertOrEhrhart + "series:" << endl << HS_Num;
+        }
+        long nr_factors = 0;
+        for (auto& it : HS_Denom) {
+            nr_factors += it.second;
+        }
+        out << "denominator with " << nr_factors << " factors:" << endl;
+        out << HS_Denom;
+        out << endl;
+        if (HS.getShift() != 0) {
+            out << "shift = " << HS.getShift() << endl << endl;
+        }
     }
 
     out << "degree of " + HilbertOrEhrhart + "Series as rational function = " << HS.getDegreeAsRationalFunction() << endl << endl;
-    if (v_is_symmetric(HS_Num)) {
+    if(write_series && v_is_symmetric(HS_Num)) {
         out << "The numerator of the " + HilbertOrEhrhart + "series is symmetric." << endl << endl;
     }
-    if (HS.get_expansion_degree() > -1) {
+    if (write_series && HS.get_expansion_degree() > -1) {
         vector<mpz_class> expansion = HS.getExpansion();
         out << "Expansion of " + HilbertOrEhrhart + "series" << endl;
         for (size_t i = 0; i < expansion.size(); ++i)
             out << i + HS.getShift() << ": " << expansion[i] << endl;
         out << endl;
     }
-    long period = HS.getPeriod();
-    if (period == 1 && (HS_Denom.size() == 0 || HS_Denom.begin()->first == (long)HS_Denom.size())) {
+    if (write_polynomial) {
         out << HilbertOrEhrhart + "polynomial:" << endl;
+           if (HS.get_nr_coeff_quasipol() >= 0) {
+                out << "only " << HS.get_nr_coeff_quasipol() << " highest coefficients computed" << endl;
+                // out << "their common period is " << HS.getHilbertQuasiPolynomial().size() << "" << endl;
+        }
         out << HS.getHilbertQuasiPolynomial()[0];
         out << "with common denominator = ";
         out << HS.getHilbertQuasiPolynomialDenom();
         out << endl << endl;
     }
-    else {
+    if(write_cyclo) {
         // output cyclonomic representation
         out << HilbertOrEhrhart << "series with cyclotomic denominator:" << endl;
         out << HS.getCyclotomicNum();
         out << "cyclotomic denominator:" << endl;
         out << HS.getCyclotomicDenom();
         out << endl;
-        // Hilbert quasi-polynomial
+    }
+
+    if(write_quasi_pol){
         HS.computeHilbertQuasiPolynomial();
         if (HS.isHilbertQuasiPolynomialComputed()) {
             out << HilbertOrEhrhart + "quasi-polynomial of period " << period << ":" << endl;
@@ -1201,10 +1273,49 @@ void Output<Integer>::writeSeries(ofstream& out, const HilbertSeries& HS, string
     }
 }
 
+template <typename Integer>
+void Output<Integer>::write_induction_matrices() {
+
+    string name_open = name + ".ind";
+    ofstream ind_out(name_open);
+    write_vec_vec_Mat(Result->getInductionMatrices(), ind_out);
+}
+
 //---------------------------------------------------------------------------
 
 template <typename Integer>
 void Output<Integer>::write_files() {
+
+    if(Result->isComputed(ConeProperty::InductionMatrices))
+        write_induction_matrices();
+
+    if(Result->isComputed(ConeProperty::SingleFusionRing)){
+        write_single_fusion_file(Result->getFusionBasicCone(), name, Result->getEmbeddingDim(), Result->getSingleFusionRing(), no_matrices_output);
+        return;
+    }
+
+    if(Result->isComputed(ConeProperty::FusionRings) || Result->isComputed(ConeProperty::SimpleFusionRings)){
+        if(Result->isComputed(ConeProperty::FusionRings)){
+            write_fusion_files(Result->getFusionBasicCone(), name, Result->isComputed(ConeProperty::SimpleFusionRings),
+                            Result->isComputed(ConeProperty::NonsimpleFusionRings), Result->getEmbeddingDim(),
+                            Result->getSimpleFusionRingsMatrix(), Result->getNonsimpleFusionRingsMatrix(),
+                            no_matrices_output, false);
+        }
+        else{ // only soimple computed
+            Matrix<Integer> Zero;
+            write_fusion_files(Result->getFusionBasicCone(),name, Result->isComputed(ConeProperty::SimpleFusionRings),
+                            Result->isComputed(ConeProperty::NonsimpleFusionRings), Result->getEmbeddingDim(),
+                            Result->getSimpleFusionRingsMatrix(), Zero,
+                            no_matrices_output,false);
+        }
+        return;
+    }
+
+    if(Result->isComputed(ConeProperty::ModularGradings)){
+        write_modular_gradings(name, Result->getModularGradings());
+        return;
+    }
+
     size_t i, nr;
     vector<libnormaliz::key_t> rees_ideal_key;
 
@@ -1277,6 +1388,14 @@ void Output<Integer>::write_files() {
         write_locus("fac", Result->getDualFaceLattice(),"dual");
     }
 
+    if(Result->isComputed(ConeProperty::FaceLatticeOrbits)) {  // write face lattice
+        write_locus("fac", Result->getFaceLatticeOrbits(),"primal_orbits");
+    }
+
+    if (Result->isComputed(ConeProperty::DualFaceLatticeOrbits)) {  // write dual face lattice
+        write_locus("fac", Result->getDualFaceLatticeOrbits(),"dual_orbits");
+    }
+
     if (Result->isComputed(ConeProperty::SingularLocus)) {  // write dual face lattice
         write_locus("sng", Result->getSingularLocus(),"");
     }
@@ -1306,8 +1425,21 @@ void Output<Integer>::write_files() {
             nr_orig_gens = Result->getNrOriginalMonoidGenerators();
             out << nr_orig_gens << " original generators of the toric ring" << endl;
         }
-        if (!homogeneous && Result->isComputed(ConeProperty::NumberLatticePoints) && !Result->isIntHullCone()) {
-            out << Result->getNumberLatticePoints() << module_generators_name << endl;
+        size_t nr_lattice_points = 0;
+        bool print_nr_of_allice_points = false;;
+        if(Result->isComputed(ConeProperty::NumberLatticePoints)){
+            nr_lattice_points = Result->getNumberLatticePoints();
+            print_nr_of_allice_points = true;
+        }
+        else{
+            if(Result->isComputed(ConeProperty::SingleLatticePoint)){
+                print_nr_of_allice_points = true;
+                if (Result->getSingleLatticePoint().size() > 0)
+                    nr_lattice_points = 1;
+            }
+        }
+        if (!homogeneous && print_nr_of_allice_points && !Result->isIntHullCone()) {
+            out << nr_lattice_points << module_generators_name << endl;
         }
         if (Result->isComputed(ConeProperty::HilbertBasis) && !Result->isIntHullCone()) {
             out << Result->getNrHilbertBasis() << " Hilbert basis elements" << of_monoid << endl;
@@ -1350,6 +1482,18 @@ void Output<Integer>::write_files() {
                 trunc = " (possibly truncated)";
             out << "dual f-vector" << trunc << ":" << endl << Result->getDualFVector() << endl;
         }
+        if (Result->isComputed(ConeProperty::FVectorOrbits)) {
+            string trunc = "";
+            if (Result->getFVectorOrbits()[0] != 1)
+                trunc = " (possibly truncated)";
+            out << "f-vector orbits" << trunc << ":" << endl << Result->getFVectorOrbits() << endl;
+        }
+        if (Result->isComputed(ConeProperty::DualFVectorOrbits)) {
+            string trunc = "";
+            if (Result->getDualFVectorOrbits()[0] != 1)
+                trunc = " (possibly truncated)";
+            out << "dual f-vector orbits" << trunc << ":" << endl << Result->getDualFVectorOrbits() << endl;
+        }
         if (Result->isComputed(ConeProperty::ExcludedFaces)) {
             out << Result->getNrExcludedFaces() << " excluded faces" << endl;
             out << endl;
@@ -1402,10 +1546,10 @@ void Output<Integer>::write_files() {
         }
         if (homogeneous && Result->isComputed(ConeProperty::IsSerreR1)) {
             if (Result->isIntegrallyClosed()) {
-                out << "original monoid satifies Serre cindition R1" << endl;
+                out << "original monoid satisfies Serre condition R1" << endl;
             }
             else {
-                out << "original monoid vioaltes Serre cindition R1" << endl;
+                out << "original monoid violates Serre condition R1" << endl;
             }
         }
         out << endl;
@@ -1593,9 +1737,12 @@ void Output<Integer>::write_files() {
         }
 
        if (Result->isComputed(ConeProperty::SingleLatticePoint)) {
+           // output can be suppressed in tests because non-uniqueness of the lattice point
             if (Result->getSingleLatticePoint().size() > 0) {
-                out << "Lattice_point:" << endl;
-                out <<  Result->getSingleLatticePoint();
+                if(!no_matrices_output){
+                    out << "Lattice point:" << endl;
+                    out <<  Result->getSingleLatticePoint();
+                }
             }
             else
                 out << "No lattice point found" << endl;
@@ -1850,6 +1997,138 @@ void Output<Integer>::write_files() {
     write_Stanley_dec();
 }
 
+void write_modular_gradings(const string& name, const vector<vector<dynamic_bitset> >& modular_gradings){
+
+    string name_open = name + ".out";  // preparing output files
+    ofstream out(name_open);
+    out << modular_gradings.size() << " modular gradings" << endl;
+    out << endl;
+    out << "***********************************************************************" << endl << endl;
+    out << modular_gradings.size() << " modular gradings:" << endl;
+    for(size_t i = 0; i < modular_gradings.size(); ++i){
+
+        out << "modular grading " << i + 1 << endl << endl;
+        for(auto& p: modular_gradings[i])
+            out << bitset_to_key(p);
+        out << "---------------------" << endl;
+    }
+}
+
+template <typename Integer>
+void write_single_fusion_file(const FusionBasic fusion_basic, const string& name,
+                        size_t embdim, vector<Integer> SingleFusionRing,
+                        const bool no_matrices_output) {
+
+    Matrix<Integer> Simple, Nonsimple;
+    Matrix<Integer> Fusion;
+    if(SingleFusionRing.size() > 0){
+        Fusion.resize(0,SingleFusionRing.size());
+        Fusion.append(SingleFusionRing);
+
+    }
+
+    split_into_simple_and_nonsimple(fusion_basic, Simple, Nonsimple, Fusion, libnormaliz::verbose);
+
+    write_fusion_files(fusion_basic, name, true,true, embdim,Simple, Nonsimple,no_matrices_output, true);
+
+    return;
+}
+
+template <typename Integer>
+void write_fusion_files(const FusionBasic fusion_basic, const string& name, const bool simple_fusion_rings,
+                        const bool non_simple_fusion_rings,
+                        size_t embdim, const Matrix<Integer>& SimpleFusionRings,
+                        const Matrix<Integer>& NonsimpleFusionRings,
+                        const bool no_matrices_output, const bool only_one) {
+
+    string name_open = name + ".out";  // preparing output files
+    const char* file = name_open.c_str();
+    ofstream out(file);
+    if (out.fail()) {
+        throw BadInputException("Cannot write to output file. Typo in directory name?");
+    }
+
+    FusionComp<Integer> fusion(fusion_basic);
+
+    string simple_fusion_text, nonsimple_fusion_text;
+
+    if(fusion.candidate_given){
+        simple_fusion_text =  " fusion rings not containing candidate subring";
+        nonsimple_fusion_text = " fusion rings containing candidate subring";
+    }
+    else{
+        simple_fusion_text =  " simple fusion rings up to isomorphism";
+        nonsimple_fusion_text = " nonsimple fusion rings up to isomorphism";
+    }
+
+    bool fusion_rings = simple_fusion_rings && non_simple_fusion_rings;
+    size_t total_nr_fusion_rings = NonsimpleFusionRings.nr_of_rows() + SimpleFusionRings.nr_of_rows();
+    if(fusion_rings){
+        if(only_one && total_nr_fusion_rings > 0)
+            out << total_nr_fusion_rings << " fusion rings up to isomorphism (only single fusion ring  asked for)" << endl;
+        else
+            out << total_nr_fusion_rings << " fusion rings up to isomorphism" << endl;
+    }
+    if(simple_fusion_rings){
+        out << SimpleFusionRings.nr_of_rows() << simple_fusion_text << endl;
+    }
+    if(non_simple_fusion_rings){
+        out << NonsimpleFusionRings.nr_of_rows() << nonsimple_fusion_text << endl;
+    }
+
+    out << endl;
+
+    if(embdim == 0){
+        embdim = NonsimpleFusionRings.nr_of_columns();
+    }
+
+    if(embdim == 0){
+        embdim = SimpleFusionRings.nr_of_columns();
+    }
+
+    if(embdim > 0){
+
+        vector<Integer> dehom(embdim);
+        dehom.back() = 1;
+        out << "Embedding dimension = " << embdim << endl;
+        out << endl;
+        out << "dehomogenization" << endl;
+        out << dehom;
+    }
+
+    out << endl;
+    out << "***********************************************************************" << endl << endl;
+
+    if(no_matrices_output){
+        out.close();
+        return;
+    }
+
+    if(simple_fusion_rings){
+        out <<  SimpleFusionRings.nr_of_rows() << simple_fusion_text << ":" << endl;
+        SimpleFusionRings.pretty_print(out);
+        out << endl;
+    }
+
+    if(non_simple_fusion_rings){
+        out << NonsimpleFusionRings.nr_of_rows() << nonsimple_fusion_text << ":" << endl;
+        NonsimpleFusionRings.pretty_print(out);
+        out << endl;
+    }
+    out.close();
+
+    if(write_fusion_mult_tables_from_input){
+        name_open = name + ".fus";
+        ofstream ftb_out(name_open);
+        Matrix<Integer> AllFusionRings = SimpleFusionRings;
+        if(NonsimpleFusionRings.nr_of_rows() > 0)
+            AllFusionRings.append(NonsimpleFusionRings);
+        fusion.write_all_data_tables(AllFusionRings, ftb_out);
+        ftb_out.close();
+    }
+
+}
+
 #ifndef NMZ_MIC_OFFLOAD  // offload with long is not supported
 template class Output<long>;
 #endif
@@ -1860,4 +2139,42 @@ template class Output<mpz_class>;
 template class Output<renf_elem_class>;
 #endif
 
+template void write_fusion_files(const FusionBasic fusion_basic, const string& name, const bool non_simple_fusion_rings,
+                                 const bool simple_fusion_rings,
+                                 size_t embdim, const Matrix<long long>& SimpleFusionRings,
+                                 const Matrix<long long>& NonSimpleFusionRings,
+                                 const bool no_matrices_output, const bool only_one);
+
+template void write_fusion_files(const FusionBasic fusion_basic, const string& name, const bool non_simple_fusion_rings,
+                                 const bool simple_fusion_rings,
+                                 size_t embdim, const Matrix<long>& SimpleFusionRings,
+                                 const Matrix<long>& NonSimpleFusionRings,
+                                 const bool no_matrices_output, const bool only_one);
+
+template void write_fusion_files(const FusionBasic fusion_basic, const string& name, const bool non_simple_fusion_rings,
+                                 const bool simple_fusion_rings,
+                                 size_t embdim, const Matrix<mpz_class>& SimpleFusionRings,
+                                 const Matrix<mpz_class>& NonSimpleFusionRings,
+                                 const bool no_matrices_output, const bool only_one);
+#ifdef ENFNORMALIZ
+template void write_fusion_files(const FusionBasic fusion_basic, const string& name, const bool non_simple_fusion_rings,
+                                 const bool simple_fusion_rings,size_t embdim,
+                                 const Matrix<renf_elem_class>& SimpleFusionRings,
+                                 const Matrix<renf_elem_class>& NonSimpleFusionRings,
+                                 const bool no_matrices_output, const bool only_one);
+#endif
+
+template void write_single_fusion_file(const FusionBasic fusion_basic, const string& name,
+                        size_t embdim, vector<long> SingleFusionRing, const bool no_matrices_output);
+
+template void write_single_fusion_file(const FusionBasic fusion_basic, const string& name,
+                        size_t embdim, vector<long long> SingleFusionRing, const bool no_matrices_output);
+
+template void write_single_fusion_file(const FusionBasic fusion_basic, const string& name,
+                        size_t embdim, vector<mpz_class> SingleFusionRing, const bool no_matrices_output);
+#ifdef ENFNORMALIZ
+template void write_single_fusion_file(const FusionBasic fusion_basic, const string& name,
+                        size_t embdim, vector<renf_elem_class> SingleFusionRing, const bool no_matrices_output);
+#endif
+
 }  // namespace libnormaliz
